using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using FitConnect.Interfaces;
using FitConnect.Models;
using FitConnect.Services;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;

namespace FitConnect;

/// <summary>
///     Fluent API for the FitConnect.Subscriber
/// </summary>
public class FluentSubscriber : Subscriber,
    IFluentSubscriber,
    IFluentSubscriberWithSubmission {
    private OAuthAccessToken? _token;


    public FluentSubscriber(FitConnectEndpoints endpoints,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService,
        X509Certificate2 certificate,
        ILogger? logger = null) : base(endpoints, oAuthService, submissionService,
        routeService, logger) {
    }


    /// <summary>
    /// </summary>
    /// <param name="destinationId"></param>
    /// <param name="skip"></param>
    /// <param name="take"></param>
    /// <returns></returns>
    public IEnumerable<SubmissionForPickupDto> GetAvailableSubmissions(string? destinationId = null,
        int skip = 0,
        int take = 100) {
        if (_token == null)
            Authenticate(Owner.ClientId, Owner.ClientSecret);

        var submissionsResult = SubmissionService.ListSubmissions(destinationId, 0, 100).Result;

        // Creating a dictionary of destinationId to submissionIds from the REST API result
        return submissionsResult.Submissions;
    }


    /// <summary>
    ///     Receives a specific submission from the server.
    /// </summary>
    /// <param name="submissionId"></param>
    /// <returns></returns>
    public IFluentSubscriberWithSubmission RequestSubmission(string submissionId) {
        if (_token == null)
            Authenticate(Owner.ClientId, Owner.ClientSecret);

        var submission = (Submission)SubmissionService.GetSubmission(submissionId);
        var metaDataString = Encryption.DecryptData(submission.EncryptedMetadata);
        if (metaDataString != null)
            submission.Metadata =
                JsonConvert.DeserializeObject<Metadata>(metaDataString);

        var dataString = Encryption.DecryptData(submission.EncryptedData);
        if (dataString != null)
            submission.Data =
                JsonConvert.DeserializeObject<Data>(dataString);
        Submission = submission;
        return this;
    }

    public Submission Submission { get; private set; }


    /// <summary>
    ///     Reading attachments for a submission.
    /// </summary>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public IFluentSubscriberWithSubmission GetAttachments(
        Func<IEnumerable<Attachment>, bool> canConfirmSubmission) {
        // TODO add guard calls

        var attachments = new List<Attachment>();
        foreach (var id in Submission!.Attachments.Select(a => a.Id)) {
            var encryptedAttachment = SubmissionService.GetAttachment(Submission.Id, id);

            var content = Encryption.DecryptData(Convert.FromBase64String(encryptedAttachment));

            // TODO where do I get the hash from the server to verify the attachment?
            var hash = MD5.Create(HashAlgorithmName.SHA512.ToString())?.ComputeHash(content) ??
                       Array.Empty<byte>();

            attachments.Add(new Attachment(id, content, Convert.ToBase64String(hash), "dummy.pdf"));
        }

        Submission.Attachments = attachments;

        if (canConfirmSubmission(attachments)) ConfirmSubmission();

        return this;
    }

    /// <summary>
    /// </summary>
    /// <param name="clientId"></param>
    /// <param name="clientSecret"></param>
    /// <param name="scope"></param>
    /// <returns></returns>
    private FluentSubscriber
        Authenticate(string clientId, string clientSecret, string? scope = null) {
        _token ??= AuthenticateAsync(clientId, clientSecret, scope).Result;

        return this;
    }


    private FluentSubscriber ConfirmSubmission() {
        // TODO add guard calls
        return this;
        throw new NotImplementedException();
        SubmissionService.SubmitSubmission(Submission.Id, (SubmitSubmissionDto)Submission);

        return this;
    }
}
